"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var _ = require("lodash");
var bean_utils_1 = require("hornet-js-bean/src/bean-utils");
var hornet_db_connector_1 = require("hornet-js-database/src/sequelize/hornet-db-connector");
var HornetGenericDAO = /** @class */ (function (_super) {
    tslib_1.__extends(HornetGenericDAO, _super);
    function HornetGenericDAO(entity, modelDAO) {
        var _this = _super.call(this, modelDAO) || this;
        _this.classEntity = entity;
        _this.classEntity["idName"] = entity["primaryKeyAttribute"];
        _this.entity = _this.modelDAO[_this.classEntity["entityName"]];
        return _this;
    }
    /**
     * Méthode d'insertion de données reposant sur le create de Sequelize
     * @param data Données pour créer l'instance
     */
    HornetGenericDAO.prototype.insertGeneric = function (data) {
        return this.modelDAO[this.classEntity["entityName"]]
            .create(data);
    };
    /**
     * Méthode de sélection d'un ensemble de données reposant sur le
     * findAll de Sequelize
     * M est un type dynamique lié au paramètre bean indiquant le type retour
     * contenu dans la promesse
     * @param criteres : critères de sélections
     * @param bean : Type dans lequel convertir les données retournées
     */
    HornetGenericDAO.prototype.findAllGeneric = function (criteres, bean) {
        if (criteres) {
            var p = this.modelDAO[this.classEntity["entityName"]]
                .findAll(this.getQueryObject(criteres));
            if (bean) {
                return p.then(function (results) {
                    return bean_utils_1.BeanUtils.mapArray(bean, results);
                });
            }
            else {
                return p;
            }
        }
        else {
            var p = this.modelDAO[this.classEntity["entityName"]].findAll();
            if (bean) {
                return p.then(function (results) {
                    return bean_utils_1.BeanUtils.mapArray(bean, results);
                });
            }
            else {
                return p;
            }
        }
    };
    /**
     * Méthode de sélection d'une instance reposant sur le
     * findOne de Sequelize
     * M est un type dynamique lié au paramètre bean indiquant le type retour
     * contenu dans la promesse
     * @param id : identifiant critère de sélection
     * @param bean : Type dans lequel convertir l'instance retournée
     */
    HornetGenericDAO.prototype.findByIdGeneric = function (id, bean) {
        var p = this.modelDAO[this.classEntity["entityName"]]
            .findOne({ where: this.computeId(id) });
        if (bean) {
            return p.then(function (result) {
                return bean_utils_1.BeanUtils.mapObject(bean, result);
            });
        }
        else {
            return p;
        }
    };
    /**
     * Méthode de suppression reposant sur le destroy de Sequelize
     * @param id : identifiant ou ensemble d'identifiant à supprimer
     */
    HornetGenericDAO.prototype.deleteByIdGeneric = function (id) {
        return this.modelDAO[this.classEntity["entityName"]]
            .destroy({ where: this.computeId(id) });
    };
    /**
     * Méthode de modification reposant sur l'update de Sequelize
     * @param id : identifiant ou ensemble d'identifiant à modifier
     */
    HornetGenericDAO.prototype.updateByIdGeneric = function (id, data) {
        return this.modelDAO[this.classEntity["entityName"]]
            .update(data, { where: this.computeId(id) });
    };
    HornetGenericDAO.prototype.computeId = function (id) {
        var where = {};
        where[this.classEntity["idName"]] = id;
        return where;
    };
    /**
     * Transforme un Paginate en conf pagination/limiting pour Sequelize
     * @param paginate Configuration de type Paginate
     */
    HornetGenericDAO.prototype.getPaginationConf = function (paginate) {
        var offset = 0;
        if (paginate.pageIndex > 1) {
            offset = paginate.pageIndex * paginate.itemsPerPage;
        }
        var limit = paginate.itemsPerPage;
        return { offset: offset, limit: limit };
    };
    /**
     * Transforme un tableau d'attributs en liste d'attributs
     * à requêter par Sequelize
     * @param attributes
     */
    HornetGenericDAO.prototype.getAttributesConf = function (attributes) {
        return { attributes: attributes };
    };
    /**
     * Utilise les informations fournies pour préparer les critères
     * de la requête utilisés par Sequelize
     * @param where
     */
    HornetGenericDAO.prototype.getWhereConf = function (where) {
        return { where: where };
    };
    HornetGenericDAO.prototype.getIncludeConf = function (include) {
        return { include: include };
    };
    HornetGenericDAO.prototype.getOrderConf = function (order) {
        return { order: order };
    };
    /**
     * Construit l'objet passé à la méthode de requêtage sequelize
     * @param criteres
     */
    HornetGenericDAO.prototype.getQueryObject = function (criteres) {
        var queryObject = {};
        if (criteres.attributes) {
            queryObject = _.assignIn(queryObject, this.getAttributesConf(criteres.attributes));
        }
        if (criteres.paginate) {
            queryObject = _.assignIn(queryObject, this.getPaginationConf(criteres.paginate));
        }
        if (criteres.where) {
            queryObject = _.assignIn(queryObject, this.getWhereConf(criteres.where));
        }
        if (criteres.include) {
            queryObject = _.assignIn(queryObject, this.getIncludeConf(criteres.include));
        }
        if (criteres.order) {
            queryObject = _.assignIn(queryObject, this.getOrderConf(criteres.order));
        }
        return queryObject;
    };
    return HornetGenericDAO;
}(hornet_db_connector_1.HornetDbConnector));
exports.HornetGenericDAO = HornetGenericDAO;

//# sourceMappingURL=hornet-generic-dao.js.map
